@@ -42,12 +42,14 @@ struct _EBookBackendMAPIGALPrivate
 	gboolean is_summary_ready;
 	gboolean is_writable;
 	gchar *uri;
-	gchar *book_name;
 
 	GMutex *lock;
 	gchar *summary_file_name;
 	EBookBackendSummary *summary;
 	EBookBackendCache *cache;
+
+	GStaticMutex running_mutex;
+	GHashTable *view_to_closure_hash; /* EDataBookView -> BESearchClosure */
 };
 
 #define SUMMARY_FLUSH_TIMEOUT 5000
@@ -194,12 +196,6 @@ build_cache (EBookBackendMAPIGAL *ebmapi)
 		priv->cache = e_book_backend_cache_new (priv->uri);
 	}
 
-	if (!priv->summary) {
-		priv->summary = e_book_backend_summary_new (priv->summary_file_name,
-							    SUMMARY_FLUSH_TIMEOUT);
-		printf("Summary file name is %s\n", priv->summary_file_name);
-	}
-
 	fgd.ebmapi = ebmapi;
 	fgd.book_view = find_book_view (ebmapi);
 	fgd.fid = exchange_mapi_connection_get_default_folder_id (priv->conn, olFolderContacts);
@@ -211,7 +207,7 @@ build_cache (EBookBackendMAPIGAL *ebmapi)
 					fetch_gal_cb, &fgd);
 
 	if (fgd.book_view) {
-		e_data_book_view_notify_complete (fgd.book_view, GNOME_Evolution_Addressbook_Success);
+		e_data_book_view_notify_complete (fgd.book_view, NULL /* Success */);
 		e_data_book_view_unref (fgd.book_view);
 	}
 
@@ -241,7 +237,7 @@ e_book_backend_mapi_gal_get_supported_fields (EBookBackend *backend,
 	fields = mapi_book_utils_get_supported_fields ();
 	e_data_book_respond_get_supported_fields (book,
 						  opid,
-						  GNOME_Evolution_Addressbook_Success,
+						  NULL /* Success */,
 						  fields);
 	g_list_free (fields);
 }
@@ -256,7 +252,7 @@ e_book_backend_mapi_gal_get_required_fields (EBookBackend *backend,
 	fields = g_list_append (fields, (gchar *) e_contact_field_name (E_CONTACT_FILE_AS));
 	e_data_book_respond_get_required_fields (book,
 						  opid,
-						  GNOME_Evolution_Addressbook_Success,
+						  NULL /* Success */,
 						  fields);
 	g_list_free (fields);
 }
@@ -277,14 +273,15 @@ e_book_backend_mapi_gal_authenticate_user (EBookBackend *backend,
 	}
 
 	switch (priv->mode) {
-	case GNOME_Evolution_Addressbook_MODE_LOCAL:
+	case E_DATA_BOOK_MODE_LOCAL:
 		e_book_backend_notify_writable (backend, FALSE);
 		e_book_backend_set_is_writable (E_BOOK_BACKEND(backend), FALSE);
 		e_book_backend_notify_connection_status (backend, FALSE);
-		e_data_book_respond_authenticate_user (book, opid, GNOME_Evolution_Addressbook_Success);
+		e_data_book_respond_authenticate_user (book, opid, NULL /* Success */);
 		return;
 
-	case GNOME_Evolution_Addressbook_MODE_REMOTE:
+	case E_DATA_BOOK_MODE_REMOTE:
+		g_static_mutex_lock (&priv->running_mutex);
 
 		/* rather reuse already established connection */
 		priv->conn = exchange_mapi_connection_find (priv->profile);
@@ -293,8 +290,11 @@ e_book_backend_mapi_gal_authenticate_user (EBookBackend *backend,
 		else if (!priv->conn)
 			priv->conn = exchange_mapi_connection_new (priv->profile, passwd);
 
-		if (!priv->conn)
-			return e_data_book_respond_authenticate_user (book, opid,GNOME_Evolution_Addressbook_OtherError);
+		if (!priv->conn) {
+			e_data_book_respond_authenticate_user (book, opid, EDB_ERROR_EX (OTHER_ERROR, "Cannot connect"));
+			g_static_mutex_unlock (&priv->running_mutex);
+			return;
+		}
 
 		if (priv->cache && priv->is_cache_ready) {
 			printf("FIXME: Should check for an update in the cache\n");
@@ -309,7 +309,8 @@ e_book_backend_mapi_gal_authenticate_user (EBookBackend *backend,
 			}
 		}
 		e_book_backend_set_is_writable (backend, FALSE);
-		e_data_book_respond_authenticate_user (book, opid, GNOME_Evolution_Addressbook_Success);
+		e_data_book_respond_authenticate_user (book, opid, NULL /* Success */);
+		g_static_mutex_unlock (&priv->running_mutex);
 		return;
 
 	default :
@@ -350,7 +351,7 @@ e_book_backend_mapi_gal_create_contact (EBookBackend *backend,
 		const gchar   *vcard)
 {
 	e_data_book_respond_create (book, opid,
-				    GNOME_Evolution_Addressbook_PermissionDenied,
+				    EDB_ERROR (PERMISSION_DENIED),
 				    NULL);
 }
 
@@ -361,7 +362,7 @@ e_book_backend_mapi_gal_remove_contacts (EBookBackend *backend,
 		 GList        *ids)
 {
 	e_data_book_respond_remove_contacts (book, opid,
-					     GNOME_Evolution_Addressbook_PermissionDenied,
+					     EDB_ERROR (PERMISSION_DENIED),
 					     NULL);
 }
 
@@ -372,25 +373,24 @@ e_book_backend_mapi_gal_modify_contact (EBookBackend *backend,
 		const gchar   *vcard)
 {
 	e_data_book_respond_modify (book, opid,
-				    GNOME_Evolution_Addressbook_PermissionDenied,
+				    EDB_ERROR (PERMISSION_DENIED),
 				    NULL);
 }
 
-static GNOME_Evolution_Addressbook_CallStatus
+static void
 e_book_backend_mapi_gal_load_source (EBookBackend *backend,
 				 ESource      *source,
-				 gboolean     only_if_exists)
+				 gboolean     only_if_exists,
+				 GError **perror)
 {
 	EBookBackendMAPIGALPrivate *priv = ((EBookBackendMAPIGAL *) backend)->priv;
 	const gchar *offline, *tmp;
-	gchar **tokens;
-	gchar *uri = NULL;
 
 	if (enable_debug)
 		printf("MAPI load source\n");
 
 	if (e_book_backend_is_loaded (backend))
-		return GNOME_Evolution_Addressbook_Success;
+		return /* Success */;
 
 	offline = e_source_get_property (source, "offline_sync");
 	if (offline  && g_str_equal (offline, "1"))
@@ -398,29 +398,21 @@ e_book_backend_mapi_gal_load_source (EBookBackend *backend,
 
 	/* Either we are in Online mode or this is marked for offline */
 
-	priv->uri = g_strdup (e_source_get_uri (source));
+	priv->uri = e_source_get_uri (source);
 
-	tokens = g_strsplit (priv->uri, ";", 2);
-	if (tokens[0])
-		uri = g_strdup (tokens [0]);
-	priv->book_name  = g_strdup (tokens[1]);
-	if (priv->book_name == NULL) {
-		g_warning ("Bookname is null for %s\n", uri);
-		return GNOME_Evolution_Addressbook_OtherError;
-	}
-	g_strfreev (tokens);
-
-	if (priv->mode ==  GNOME_Evolution_Addressbook_MODE_LOCAL &&
+	if (priv->mode ==  E_DATA_BOOK_MODE_LOCAL &&
 	    !priv->marked_for_offline ) {
-		return GNOME_Evolution_Addressbook_OfflineUnavailable;
+		g_propagate_error (perror, EDB_ERROR (OFFLINE_UNAVAILABLE));
+		return;
 	}
 
+	g_free (priv->summary_file_name);
+	priv->summary_file_name = get_filename_from_uri (priv->uri, "cache.summary");
+	if (priv->summary) g_object_unref (priv->summary);
+	priv->summary = e_book_backend_summary_new (priv->summary_file_name, SUMMARY_FLUSH_TIMEOUT);
+
 	if (priv->marked_for_offline) {
-		priv->summary_file_name = get_filename_from_uri (priv->uri, "cache.summary");
 		if (g_file_test (priv->summary_file_name, G_FILE_TEST_EXISTS)) {
-			printf("Loading the summary\n");
-			priv->summary = e_book_backend_summary_new (priv->summary_file_name,
-								    SUMMARY_FLUSH_TIMEOUT);
 			e_book_backend_summary_load (priv->summary);
 			priv->is_summary_ready = TRUE;
 		}
@@ -429,7 +421,6 @@ e_book_backend_mapi_gal_load_source (EBookBackend *backend,
 		if (e_book_backend_cache_exists (priv->uri)) {
 			gchar *last_time;
 
-			printf("Loading the cache\n");
 			priv->cache = e_book_backend_cache_new (priv->uri);
 
 			last_time = e_book_backend_cache_get_time (priv->cache);
@@ -437,20 +428,18 @@ e_book_backend_mapi_gal_load_source (EBookBackend *backend,
 			g_free (last_time);
 		}
 		//FIXME: We may have to do a time based reload. Or deltas should upload.
-	} else {
-		priv->summary = e_book_backend_summary_new (NULL,SUMMARY_FLUSH_TIMEOUT);
 	}
 
-	g_free (uri);
 	e_book_backend_set_is_loaded (E_BOOK_BACKEND (backend), TRUE);
 	e_book_backend_set_is_writable (backend, FALSE);
-	if (priv->mode ==  GNOME_Evolution_Addressbook_MODE_LOCAL) {
+	if (priv->mode ==  E_DATA_BOOK_MODE_LOCAL) {
 		e_book_backend_set_is_writable (backend, FALSE);
 		e_book_backend_notify_writable (backend, FALSE);
 		e_book_backend_notify_connection_status (backend, FALSE);
 		if (!priv->cache) {
 			printf("Unfortunately the cache is not yet created\n");
-			return GNOME_Evolution_Addressbook_OfflineUnavailable;
+			g_propagate_error (perror, EDB_ERROR (OFFLINE_UNAVAILABLE));
+			return;
 		}
 	} else {
 		e_book_backend_notify_connection_status (backend, TRUE);
@@ -462,8 +451,8 @@ e_book_backend_mapi_gal_load_source (EBookBackend *backend,
 	tmp = e_source_get_property (source, "folder-id");
 
 	/* Once aunthentication in address book works this can be removed */
-	if (priv->mode == GNOME_Evolution_Addressbook_MODE_LOCAL) {
-		return GNOME_Evolution_Addressbook_Success;
+	if (priv->mode == E_DATA_BOOK_MODE_LOCAL) {
+		return /* Success */;
 	}
 
 	// writable property will be set in authenticate_user callback
@@ -472,12 +461,10 @@ e_book_backend_mapi_gal_load_source (EBookBackend *backend,
 
 	if (enable_debug)
 		printf("For profile %s and folder %s - %016" G_GINT64_MODIFIER "X\n", priv->profile, tmp, priv->fid);
-
-	return GNOME_Evolution_Addressbook_Success;
 }
 
 static void
-e_book_backend_mapi_gal_set_mode (EBookBackend *backend, GNOME_Evolution_Addressbook_BookMode mode)
+e_book_backend_mapi_gal_set_mode (EBookBackend *backend, EDataBookMode mode)
 {
 	EBookBackendMAPIGALPrivate *priv = ((EBookBackendMAPIGAL *) backend)->priv;
 
@@ -486,13 +473,13 @@ e_book_backend_mapi_gal_set_mode (EBookBackend *backend, GNOME_Evolution_Address
 
 	priv->mode = mode;
 	if (e_book_backend_is_loaded (backend)) {
-		if (mode == GNOME_Evolution_Addressbook_MODE_LOCAL) {
+		if (mode == E_DATA_BOOK_MODE_LOCAL) {
 			e_book_backend_notify_writable (backend, FALSE);
 			e_book_backend_set_is_writable (backend, FALSE);
 			e_book_backend_notify_connection_status (backend, FALSE);
 			/* FIXME: Uninitialize mapi here. may be.*/
 		}
-		else if (mode == GNOME_Evolution_Addressbook_MODE_REMOTE) {
+		else if (mode == E_DATA_BOOK_MODE_REMOTE) {
 			e_book_backend_notify_writable (backend, FALSE);
 			e_book_backend_set_is_writable (backend, FALSE);
 			e_book_backend_notify_connection_status (backend, TRUE);
@@ -501,42 +488,58 @@ e_book_backend_mapi_gal_set_mode (EBookBackend *backend, GNOME_Evolution_Address
 	}
 }
 
-static void
-e_book_backend_mapi_gal_dispose (GObject *object)
-{
-	/* FIXME : provide implmentation */
-	EBookBackendMAPIGALPrivate *priv = ((EBookBackendMAPIGAL *) object)->priv;
-
-	if (priv->profile) {
-		g_free (priv->profile);
-		priv->profile = NULL;
-	}
-	if (priv->conn) {
-		g_object_unref (priv->conn);
-		priv->conn = NULL;
-	}
-	if (priv->uri) {
-		g_free (priv->uri);
-		priv->uri = NULL;
-	}
-
-	if (priv->build_cache_thread) {
-		priv->kill_cache_build = TRUE;
-		g_thread_join (priv->build_cache_thread);
-		priv->build_cache_thread = NULL;
-	}
-}
-
 typedef struct {
 	EBookBackendMAPIGAL *bg;
-	GThread *thread;
-	EFlag *running;
+	EDataBookView *book_view;
+	gboolean stop;
 } BESearchClosure;
 
 static BESearchClosure*
-get_closure (EDataBookView *book_view)
+init_closure (EDataBookView *book_view, EBookBackendMAPIGAL *bg)
 {
-	return g_object_get_data (G_OBJECT (book_view), "closure");
+	BESearchClosure *closure;
+
+	g_return_val_if_fail (bg != NULL, NULL);
+	g_return_val_if_fail (bg->priv != NULL, NULL);
+	g_return_val_if_fail (bg->priv->view_to_closure_hash != NULL, NULL);
+	
+	closure = g_new0 (BESearchClosure, 1);
+	closure->bg = bg;
+	closure->book_view = book_view;
+	closure->stop = FALSE;
+
+	g_hash_table_insert (bg->priv->view_to_closure_hash, g_object_ref (book_view), closure);
+
+	return closure;
+}
+
+static void
+destroy_closure (BESearchClosure *closure)
+{
+	g_return_if_fail (closure != NULL);
+
+	if (closure->book_view)
+		g_object_unref (closure->book_view);
+	g_free (closure);
+}
+
+static void
+stop_book_view (EDataBookView *book_view, BESearchClosure *closure, EBookBackendMAPIGAL *mapi_backend)
+{
+	g_return_if_fail (closure != NULL);
+
+	closure->stop = TRUE;
+}
+
+static void
+untrack_book_view (EBookBackendMAPIGAL *mapi_backend, EDataBookView *book_view)
+{
+	g_return_if_fail (mapi_backend != NULL);
+	g_return_if_fail (mapi_backend->priv != NULL);
+	g_return_if_fail (mapi_backend->priv->view_to_closure_hash != NULL);
+	g_return_if_fail (book_view != NULL);
+
+	g_hash_table_remove (mapi_backend->priv->view_to_closure_hash, book_view);
 }
 
 static void
@@ -552,7 +555,7 @@ get_contacts_from_cache (EBookBackendMAPIGAL *ebmapi,
 		gchar *uid;
 		EContact *contact;
 
-                if (!e_flag_is_set (closure->running))
+                if (closure->stop)
                         break;
 
 		uid = g_ptr_array_index (ids, i);
@@ -562,9 +565,6 @@ get_contacts_from_cache (EBookBackendMAPIGAL *ebmapi,
 			g_object_unref (contact);
 		}
 	}
-	if (e_flag_is_set (closure->running))
-		e_data_book_view_notify_complete (book_view,
-						  GNOME_Evolution_Addressbook_Success);
 }
 #if 0
 static gboolean
@@ -605,97 +605,20 @@ build_restriction_emails_contains (struct mapi_SRestriction *res,
 }
 #endif
 
-static gboolean
-build_multiple_restriction_emails_contains (struct mapi_SRestriction *res,
-					    struct mapi_SRestriction_or *or_res,
-					    const gchar *query)
-{
-	gchar *email=NULL, *tmp, *tmp1;
-	//Number of restriction to apply
-	guint res_count = 6;
-
-	/* This currently supports "email foo@bar.soo" */
-	tmp = strdup (query);
-
-	tmp = strstr (tmp, "email");
-	if (tmp ) {
-		tmp = strchr (tmp, '\"');
-		if (tmp && ++tmp) {
-			tmp = strchr (tmp, '\"');
-			if (tmp && ++tmp) {
-				tmp1 = tmp;
-				tmp1 = strchr (tmp1, '\"');
-				if (tmp1) {
-					*tmp1 = 0;
-					email = tmp;
-				}
-			}
-		}
-	}
-
-	if (email==NULL || !strchr (email, '@'))
-		return FALSE;
-
-	or_res[0].rt = RES_CONTENT;
-	or_res[0].res.resContent.fuzzy = FL_FULLSTRING | FL_IGNORECASE;
-	or_res[0].res.resContent.ulPropTag = PR_EMS_AB_MANAGER_T_UNICODE;
-	or_res[0].res.resContent.lpProp.value.lpszA = email;
-
-	or_res[1].rt = RES_CONTENT;
-	or_res[1].res.resContent.fuzzy = FL_FULLSTRING | FL_IGNORECASE;
-	or_res[1].res.resContent.ulPropTag = PR_DISPLAY_NAME_UNICODE;
-	or_res[1].res.resContent.lpProp.value.lpszA = email;
-
-	or_res[2].rt = RES_CONTENT;
-	or_res[2].res.resContent.fuzzy = FL_FULLSTRING | FL_IGNORECASE;
-	or_res[2].res.resContent.ulPropTag = PR_GIVEN_NAME_UNICODE;
-	or_res[2].res.resContent.lpProp.value.lpszA = email;
-
-	or_res[3].rt = RES_CONTENT;
-	or_res[3].res.resContent.fuzzy = FL_FULLSTRING | FL_IGNORECASE;
-	or_res[3].res.resContent.ulPropTag = PROP_TAG(PT_UNICODE, 0x8084);
-	or_res[3].res.resContent.lpProp.value.lpszA = email;
-
-	or_res[4].rt = RES_CONTENT;
-	or_res[4].res.resContent.fuzzy = FL_FULLSTRING | FL_IGNORECASE;
-	or_res[4].res.resContent.ulPropTag = PROP_TAG(PT_UNICODE, 0x8094);
-	or_res[4].res.resContent.lpProp.value.lpszA = email;
-
-	or_res[5].rt = RES_CONTENT;
-	or_res[5].res.resContent.fuzzy = FL_FULLSTRING | FL_IGNORECASE;
-	or_res[5].res.resContent.ulPropTag = PROP_TAG(PT_UNICODE, 0x80a4);
-	or_res[5].res.resContent.lpProp.value.lpszA = email;
-
-	res = g_new0 (struct mapi_SRestriction, 1);
-
-	res->rt = RES_OR;
-	res->res.resOr.cRes = res_count;
-	res->res.resOr.res = or_res;
-
-	return TRUE;
-}
-
 static void
 book_view_thread (gpointer data)
 {
-	struct mapi_SRestriction res;
-	struct mapi_SRestriction_or *or_res = NULL;
-	EDataBookView *book_view = data;
-	BESearchClosure *closure = get_closure (book_view);
+	BESearchClosure *closure = data;
+	EDataBookView *book_view = closure->book_view;
 	EBookBackendMAPIGAL *backend = closure->bg;
 	EBookBackendMAPIGALPrivate *priv = backend->priv;
 	const gchar *query = NULL;
 	GPtrArray *ids = NULL;
 	GList *contacts = NULL, *temp_list = NULL;
-	//Number of multiple restriction to apply
-	guint res_count = 6;
 
 	if (enable_debug)
 		printf("mapi: book view\n");
 
-	g_object_ref (book_view);
-	e_flag_set (closure->running);
-
 	book_view_notify_status (book_view, _("Searching"));
 	query = e_data_book_view_get_card_query (book_view);
 
@@ -703,18 +626,22 @@ book_view_thread (gpointer data)
 		e_book_backend_add_book_view (E_BOOK_BACKEND (backend), book_view);
 
 	switch (priv->mode) {
-		case GNOME_Evolution_Addressbook_MODE_REMOTE:
+		case E_DATA_BOOK_MODE_REMOTE:
 			if (!priv->conn) {
+				GError *err = EDB_ERROR (AUTHENTICATION_REQUIRED);
 				e_book_backend_notify_auth_required (E_BOOK_BACKEND (backend));
-				e_data_book_view_notify_complete (book_view,
-							GNOME_Evolution_Addressbook_AuthenticationRequired);
-				g_object_unref (book_view);
+				e_data_book_view_notify_complete (book_view, err);
+				g_error_free (err);
+				untrack_book_view (backend, book_view);
+				destroy_closure (closure);
 				return;
 			}
 
 			if (priv->marked_for_offline && !priv->is_cache_ready) {
 				/* To translators : Here Evolution MAPI downloads the entries from the GAL server */
 				book_view_notify_status (book_view, _("Downloading GAL entries from server…"));
+				untrack_book_view (backend, book_view);
+				destroy_closure (closure);
 				return;
 			}
 
@@ -724,11 +651,10 @@ book_view_thread (gpointer data)
 					if (enable_debug)
 						printf ("reading the contacts from summary \n");
 					ids = e_book_backend_summary_search (priv->summary, query);
-					if (ids && ids->len > 0) {
+					if (ids && ids->len > 0)
 						get_contacts_from_cache (backend, query, ids, book_view, closure);
+					if (ids)
 						g_ptr_array_free (ids, TRUE);
-					}
-					g_object_unref (book_view);
 					break;
 				}
 
@@ -739,7 +665,7 @@ book_view_thread (gpointer data)
 								      query);
 				temp_list = contacts;
 				for (; contacts != NULL; contacts = g_list_next(contacts)) {
-					if (!e_flag_is_set (closure->running)) {
+					if (closure->stop) {
 						for (;contacts != NULL; contacts = g_list_next (contacts))
 							g_object_unref (contacts->data);
 						break;
@@ -748,58 +674,26 @@ book_view_thread (gpointer data)
 									E_CONTACT(contacts->data));
 					g_object_unref (contacts->data);
 				}
-				if (e_flag_is_set (closure->running))
-					e_data_book_view_notify_complete (book_view,
-									  GNOME_Evolution_Addressbook_Success);
 				if (temp_list)
-					 g_list_free (temp_list);
-				g_object_unref (book_view);
+					g_list_free (temp_list);
 				break;
 			}
 
 			if (e_book_backend_summary_is_summary_query (priv->summary, query)) {
-				or_res = g_new (struct mapi_SRestriction_or, res_count);
+				ids = e_book_backend_summary_search (priv->summary, query);
+				if (ids && ids->len > 0)
+					get_contacts_from_cache (backend, query, ids, book_view, closure);
+				if (ids)
+					g_ptr_array_free (ids, TRUE);
+				break;
+			}
 
-				if (!build_multiple_restriction_emails_contains (&res, or_res, query)) {
-					e_data_book_view_notify_complete (book_view,
-								  GNOME_Evolution_Addressbook_OtherError);
-					return;
-				}
-
-			if (e_flag_is_set (closure->running))
-				e_data_book_view_notify_complete (book_view,
-								  GNOME_Evolution_Addressbook_Success);
-			g_object_unref (book_view);
 			break;
-		}
 	}
 
-	if (book_view)
-		e_data_book_view_notify_complete (book_view,
-						  GNOME_Evolution_Addressbook_Success);
-	return;
-}
-
-static void
-closure_destroy (BESearchClosure *closure)
-{
-	e_flag_free (closure->running);
-	g_free (closure);
-}
-
-static BESearchClosure*
-init_closure (EDataBookView *book_view, EBookBackendMAPIGAL *bg)
-{
-	BESearchClosure *closure = g_new (BESearchClosure, 1);
-
-	closure->bg = bg;
-	closure->thread = NULL;
-	closure->running = e_flag_new ();
-
-	g_object_set_data_full (G_OBJECT (book_view), "closure",
-				closure, (GDestroyNotify)closure_destroy);
-
-	return closure;
+	e_data_book_view_notify_complete (book_view, NULL /* Success */);
+	untrack_book_view (backend, book_view);
+	destroy_closure (closure);
 }
 
 static void
@@ -811,7 +705,7 @@ e_book_backend_mapi_gal_get_contact (EBookBackend *backend,
 	if (enable_debug)
 		printf ("mapi: get contact %s\n", id);
 
-	e_data_book_respond_get_contact (book, opid, GNOME_Evolution_Addressbook_RepositoryOffline, NULL);
+	e_data_book_respond_get_contact (book, opid, EDB_ERROR (NOT_SUPPORTED), NULL);
 }
 
 static void
@@ -825,10 +719,7 @@ e_book_backend_mapi_gal_get_contact_list (EBookBackend *backend,
 	if (enable_debug)
 		printf("mapi: get contact list %s\n", query);
 
-	e_data_book_respond_get_contact_list (book, opid, GNOME_Evolution_Addressbook_RepositoryOffline,
-						      NULL);
-
-	return;
+	e_data_book_respond_get_contact_list (book, opid, EDB_ERROR (NOT_SUPPORTED), NULL);
 }
 
 static void
@@ -837,12 +728,12 @@ e_book_backend_mapi_gal_start_book_view (EBookBackend  *backend,
 {
 	BESearchClosure *closure = init_closure (book_view, E_BOOK_BACKEND_MAPIGAL (backend));
 
+	g_return_if_fail (closure != NULL);
+
 	if (enable_debug)
 		printf ("mapi: start_book_view...\n");
-	closure->thread = g_thread_create ((GThreadFunc) book_view_thread, book_view, FALSE, NULL);
-	e_flag_wait (closure->running);
 
-	/* at this point we know the book view thread is actually running */
+	g_thread_create ((GThreadFunc) book_view_thread, closure, FALSE, NULL);
 }
 
 static void
@@ -851,7 +742,8 @@ e_book_backend_mapi_gal_stop_book_view (EBookBackend  *backend,
 {
 	if (enable_debug)
 		printf("mapi: stop book view\n");
-	/* FIXME : provide implmentation */
+
+	untrack_book_view (E_BOOK_BACKEND_MAPIGAL (backend), book_view);
 }
 
 static void
@@ -860,7 +752,7 @@ e_book_backend_mapi_gal_get_changes (EBookBackend *backend, EDataBook *book, gui
 	if (enable_debug)
 		printf ("mapi: get changes\n");
 
-	e_data_book_respond_get_changes (book, opid, GNOME_Evolution_Addressbook_RepositoryOffline, NULL);
+	e_data_book_respond_get_changes (book, opid, EDB_ERROR (NOT_SUPPORTED), NULL);
 }
 
 static void
@@ -876,33 +768,105 @@ e_book_backend_mapi_gal_get_supported_auth_methods (EBookBackend *backend, EData
 	auth_methods = g_list_append (auth_methods, auth_method);
 	e_data_book_respond_get_supported_auth_methods (book,
 							opid,
-							GNOME_Evolution_Addressbook_Success,
+							NULL /* Success */,
 							auth_methods);
 	g_free (auth_method);
 	g_list_free (auth_methods);
 }
 
-static GNOME_Evolution_Addressbook_CallStatus
-e_book_backend_mapi_gal_cancel_operation (EBookBackend *backend, EDataBook *book)
+static void
+e_book_backend_mapi_gal_cancel_operation (EBookBackend *backend, EDataBook *book, GError **perror)
 {
 	if (enable_debug)
 		printf ("mapi cancel_operation...\n");
-	return GNOME_Evolution_Addressbook_CouldNotCancel;
+	g_propagate_error (perror, EDB_ERROR (COULD_NOT_CANCEL));
 }
 
 static void
 e_book_backend_mapi_gal_remove (EBookBackend *backend, EDataBook *book, guint32 opid)
 {
-	e_data_book_respond_remove (book, opid, GNOME_Evolution_Addressbook_PermissionDenied);
+	e_data_book_respond_remove (book, opid, EDB_ERROR (PERMISSION_DENIED));
 }
 
-static void e_book_backend_mapi_gal_class_init (EBookBackendMAPIGALClass *klass)
+static void
+e_book_backend_mapi_gal_init (EBookBackendMAPIGAL *backend)
+{
+	EBookBackendMAPIGALPrivate *priv;
+
+	priv = g_new0 (EBookBackendMAPIGALPrivate, 1);
+	/* Priv Struct init */
+	backend->priv = priv;
+
+	priv->build_cache_thread = NULL;
+	priv->view_to_closure_hash = g_hash_table_new (g_direct_hash, g_direct_equal);
+	g_static_mutex_init (&priv->running_mutex);
+
+/*	priv->marked_for_offline = FALSE;
+	priv->uri = NULL;
+	priv->cache = NULL;
+	priv->is_summary_ready = FALSE;
+	priv->is_cache_ready = FALSE;
+
+*/	if (g_getenv ("MAPI_DEBUG"))
+		enable_debug = TRUE;
+	else
+		enable_debug = FALSE;
+}
+
+static void
+e_book_backend_mapi_gal_dispose (GObject *object)
+{
+	EBookBackendMAPIGAL *mapi_backend = E_BOOK_BACKEND_MAPIGAL (object);
+	EBookBackendMAPIGALPrivate *priv = mapi_backend->priv;
+
+	if (priv) {
+		if (priv->view_to_closure_hash) {
+			g_hash_table_foreach (priv->view_to_closure_hash, (GHFunc) stop_book_view, mapi_backend);
+			g_hash_table_destroy (priv->view_to_closure_hash);
+			priv->view_to_closure_hash = NULL;
+		}
+
+		if (priv->build_cache_thread) {
+			priv->kill_cache_build = TRUE;
+			g_thread_join (priv->build_cache_thread);
+			priv->build_cache_thread = NULL;
+		}
+
+		#define FREE(x) if (x) { g_free (x); x = NULL; }
+		#define UNREF(x) if (x) { g_object_unref (x); x = NULL; }
+
+		/* this will also ensure any pending authentication
+		   request is finished and it's safe to free memory */
+		g_static_mutex_lock (&priv->running_mutex);
+
+		UNREF (priv->conn);
+		UNREF (priv->cache);
+		UNREF (priv->summary);
+
+		FREE (priv->profile);
+		FREE (priv->uri);
+		FREE (priv->summary_file_name);
+
+		g_static_mutex_unlock (&priv->running_mutex);
+		g_static_mutex_free (&priv->running_mutex);
+
+		FREE (mapi_backend->priv);
+
+		#undef UNREF
+		#undef FREE
+	}
+
+	/* Chain up to parent's dispose() method. */
+	if (G_OBJECT_CLASS (e_book_backend_mapi_gal_parent_class)->dispose)
+		G_OBJECT_CLASS (e_book_backend_mapi_gal_parent_class)->dispose (object);
+}
+
+static void
+e_book_backend_mapi_gal_class_init (EBookBackendMAPIGALClass *klass)
 {
 	GObjectClass  *object_class = G_OBJECT_CLASS (klass);
 	EBookBackendClass *parent_class;
 
-	e_book_backend_mapi_gal_parent_class = g_type_class_peek_parent (klass);
-
 	parent_class = E_BOOK_BACKEND_CLASS (klass);
 
 	/* Set the virtual methods. */
@@ -924,6 +888,7 @@ static void e_book_backend_mapi_gal_class_init (EBookBackendMAPIGALClass *klass)
 	parent_class->cancel_operation		 = e_book_backend_mapi_gal_cancel_operation;
 	parent_class->set_mode                   = e_book_backend_mapi_gal_set_mode;
 	parent_class->remove			 = e_book_backend_mapi_gal_remove;
+
 	object_class->dispose                    = e_book_backend_mapi_gal_dispose;
 }
 
@@ -935,32 +900,10 @@ e_book_backend_mapi_gal_new (void)
 {
 	EBookBackendMAPIGAL *backend;
 
-//	if (enable_debug)
+	if (enable_debug)
 		printf ("\ne_book_backend_mapi_gal_new...\n");
 
 	backend = g_object_new (E_TYPE_BOOK_BACKEND_MAPIGAL, NULL);
 
 	return E_BOOK_BACKEND (backend);
 }
-
-static void	e_book_backend_mapi_gal_init (EBookBackendMAPIGAL *backend)
-{
-	EBookBackendMAPIGALPrivate *priv;
-
-	priv= g_new0 (EBookBackendMAPIGALPrivate, 1);
-	/* Priv Struct init */
-	backend->priv = priv;
-
-	priv->build_cache_thread = NULL;
-
-/*	priv->marked_for_offline = FALSE;
-	priv->uri = NULL;
-	priv->cache = NULL;
-	priv->is_summary_ready = FALSE;
-	priv->is_cache_ready = FALSE;
-
-*/	if (g_getenv ("MAPI_DEBUG"))
-		enable_debug = TRUE;
-	else
-		enable_debug = FALSE;
-}
